/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
**+
** NAME:
**
**	xfpovray
**
** PURPOSE:
**
**      An XForms interface to the Persistence of Vision Raytracer v3
**
**      The basic method is as follows:  Options are set on the widget; options
**      not set are assigned default values.  All options are written out to
**      a file that is read by POV-ray upon startup.
**
**      User options in xfpovray.h 
**
** USAGE:
**
**      xfpovray [-geometry +x+y]
**
**      o Due to an XForms problem on Linux, the geometry flag cannot be
**        specified as -g.  You must type -geometry.
**      o A standard X geometry string is acceptable; however, the size
**        of the widget will be ignored; only the position can be set.
**  
** MODIFICATION HISTORY:
**
**	v0.9: 1996 July, written, mallozzir@cspar.uah.edu
**                       (http://cspar.uah.edu/~mallozzir/)
**
**      v1.0: 1996 July, RSM 
**                       o Added "Abort Render" button
**                       o Added support for Mosaic Preview     
**
**      v1.1: 1996 Sept, RSM
**                       o Code clean-up
**                       o Fixed bug with Error, Messages display 
**                       o Added -geometry option
**-
** * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */


#include <stdlib.h>

#include "forms.h"
#include "xfpovray.h"
#include "povray-defaults.h"
#include "wids_struct.h"
#include "pov_struct.h"
  
#define TIME     2
#define PUSHED   1
#define RELEASED 0
 
/* Starting position for xfpovray, default center placement */ 
int start_x;
int start_y;
int have_geometry = FALSE;
 
/* Sub-process id */ 
int povray_pid = 0;

/* Geometry for the options widgets: Do Not Edit */
const int GRP_X = 10;
const int GRP_Y = 250;
const int GRP_W = 400;
const int GRP_H = 305;

/* Callback functions */
void AbortCB();
void ClearPartialImageCB(); 
void ConfigCB();
void ContinueCB(); 
void DismissAboutCB();
void DismissMessageCB();
void DoPreviewCB();
void GetFileCB();
void HelpCB();
void PovCB();
int  PovCheckCB();
void QualityCB();
void QualityResetCB();
void QuitCB();
void RadioCB();
void SetImageSizeCB();
void SetMosaicCB();
void TogglePixmapCB();
void YesNoCB();

/* Other functions */
void write_config();
void show_image();

void pixmap_group();
void parsing_group();
void output_group();
void tracing_group();
void animation_group();
void config_group();


/* Global widget ids */
struct widget_ids wids;

/* Set options to defaults */
struct pov_options pov = {
       NULL, /* input_file_name */ 
       POV_HEIGHT,
       POV_WIDTH,
       POV_DISPLAY_DURING_RENDER,
       POV_OUTPUT_TO_FILE,
       POV_VERBOSE,
       POV_ANTIALIAS,
       POV_AA_SAMPLING_METHOD,
       POV_AA_DEPTH,
       POV_RADIOSITY,
       POV_QUALITY,
       POV_REDIRECT_ON,
       POV_VERSION, 
       POV_REMOVE_BOUNDS,
       POV_SPLIT_UNIONS,
       POV_BOUNDING,
       POV_BOUNDING_THRESHOLD,
       POV_LIGHT_BUFFER,
       POV_VISTA_BUFFER,
       POV_JITTER,
       POV_INITIAL_FRAME,
       POV_FINAL_FRAME,
       POV_CYCLIC_ANIMATION,
       POV_FIELD_RENDERING,
       POV_ODD_FIELD,
       POV_PALETTE,
       POV_OUTPUT_FILETYPE
};

/* Temporary output files */
char POV_INIFILE[1024];
char STREAM_FILE[1024];
char FATAL_FILE[1024];
char PREVIEW_FILE[1024];

int auto_view = 0;
int loaded_inifile = 0; 


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void main (int argc, char **argv)
{
 
    FL_OBJECT *obj, *ret;
    FL_IOPT control;       
    int version, revision;
    static char *home;
    char temp[10];
    int w, h;    
    
     
    /* Make sure the distribution is consistent wrt version numbers */
    if (FL_INCLUDE_VERSION != fl_library_version(&version, &revision)) {
       fprintf(stderr,"*** Header/library version mismatch!\n");
       fprintf(stderr,"    Header: %d.%d\n", FL_VERSION, FL_REVISION);
       fprintf(stderr,"    Library: %d.%d\n", version, revision);
       exit(1);
    }

    /* Check for geometry string, w (width) and h (height) are ignored */
    if (argc == 3) {
    
       if (strcmp(argv[1], "-geometry") == 0) {          
          XParseGeometry(argv[2], &start_x, &start_y, &w, &h);
          have_geometry = TRUE;
       }
       else {
          printf("xfpovray [-geometry +x+y]\n");
          return;
       }      
    }
    
    /* Get home directory, and set filepaths */
    home = getenv("HOME");
    sprintf(POV_INIFILE,  "%s/%s", home, _POV_INIFILE);
    sprintf(STREAM_FILE,  "%s/%s", home, _STREAM_FILE);
    sprintf(FATAL_FILE,   "%s/%s", home, _FATAL_FILE);
    
    /* Set some defaults and initialize the library */
    control.inputFontSize   = 12;
    control.browserFontSize = 12;
    control.buttonFontSize  = 12;
    control.labelFontSize   = 12;
    control.menuFontSize    = 12;
        
    fl_set_defaults(FL_PDInputFontSize|
                    FL_PDBrowserFontSize|
                    FL_PDButtonFontSize|
                    FL_PDLabelFontSize|
                    FL_PDMenuFontSize,
                    &control);
    
    fl_initialize (&argc, argv, "xfpovray", 0, 0);

 
    wids.main_form = fl_bgn_form(FL_UP_BOX, 420, 580);

        obj = fl_add_box(FL_UP_BOX, 5, 5, 420-10, 25, "");

	obj = fl_add_menu(FL_PUSH_MENU, 
	    10, 5, 30, 25, "File");
            fl_set_object_lstyle(obj, FL_BOLD_STYLE);
            fl_set_object_boxtype(obj, FL_NO_BOX);
            fl_set_menu(obj, "Quit");
            fl_set_object_callback(obj, QuitCB, 0);        

	obj = fl_add_menu(FL_PUSH_MENU, 
	    380, 5, 30, 25, "Help");
            fl_set_object_lstyle(obj, FL_BOLD_STYLE);
            fl_set_object_boxtype(obj, FL_NO_BOX);
            fl_set_menu(obj, "Help|About");        
            fl_set_object_callback(obj, HelpCB, 0);
                                
        obj = fl_add_frame(FL_ENGRAVED_FRAME, 10, 35, 400, 200, "");
  
        /* Left side of top widget */
	obj = fl_add_button(FL_NORMAL_BUTTON, 
	    15, 40, 100, 25, "Scene File"); 
            fl_set_object_lstyle(obj, FL_BOLD_STYLE);
            fl_set_object_callback(obj, GetFileCB, 0);

	wids.wid_file_input = fl_add_input(FL_NORMAL_INPUT, 
	    125, 40, 280, 25, "");
            fl_set_input(wids.wid_file_input, "");  

	obj = fl_add_button(FL_NORMAL_BUTTON, 
	    15, 70, 100, 25, "Edit Scene"); 
            fl_set_object_lstyle(obj, FL_BOLD_STYLE);
            fl_set_object_callback(obj, GetFileCB, 1);

	obj = fl_add_box(FL_FRAME_BOX, 
	    125, 70, 100, 25, POV_EDITOR); 
            fl_set_object_lstyle(obj, FL_BOLD_STYLE);

	obj = fl_add_button(FL_NORMAL_BUTTON, 
	    15, 100, 100, 25, "View Image"); 
            fl_set_object_lstyle(obj, FL_BOLD_STYLE);
            fl_set_object_callback(obj, GetFileCB, 2);

        wids.auto_image = fl_add_choice(FL_NORMAL_CHOICE, 
            125, 100, 100, 25, ""); 
            fl_addto_choice(wids.auto_image, "Auto OFF|Auto ON");
            fl_set_choice(wids.auto_image, auto_view + 1);
            fl_set_choice_fontstyle(wids.auto_image, FL_BOLD_STYLE); 
            fl_set_object_lstyle(wids.auto_image, FL_BOLD_STYLE);
            fl_set_object_boxtype(wids.auto_image, FL_FRAME_BOX);
            fl_set_object_callback(wids.auto_image, YesNoCB, 4);
            if (pov.display_during_render == 1) {
               fl_set_object_color(wids.auto_image, FL_COL1, FL_INACTIVE_COL); 
               fl_deactivate_object(wids.auto_image);
            }         
          
	obj = fl_add_menu(FL_PUSH_MENU, 
            15, 130, 100, 25, "Config File"); 
            fl_set_object_lstyle(obj, FL_BOLD_STYLE);
            fl_set_object_boxtype(obj, FL_UP_BOX);
            fl_set_menu(obj, "View|Save|Load");        
            fl_set_object_callback(obj, ConfigCB, 0);

 	obj = fl_add_button(FL_NORMAL_BUTTON, 
	    15, 160, 100, 55, "Render\nImage"); 
            fl_set_object_lstyle(obj, FL_BOLD_STYLE);
            fl_set_object_callback(obj, PovCB, 0);

	obj = fl_add_button(FL_NORMAL_BUTTON, 
	    125, 190, 100, 25, "Abort Render"); 
            fl_set_object_lstyle(obj, FL_BOLD_STYLE);
            fl_set_object_callback(obj, AbortCB, 0);
 
 
        /* Right side of top widget */ 
 	wids.toggle = fl_add_button(FL_NORMAL_BUTTON, 
            290, 70, 115, 25, "Options");
            fl_set_object_lstyle(wids.toggle, FL_BOLD_STYLE);
            fl_set_object_boxtype(wids.toggle, FL_NO_BOX);
            fl_set_object_callback(wids.toggle, TogglePixmapCB, 0);

        /* Option radio buttons */
        fl_bgn_group();
        
            wids.opt[0] = fl_add_lightbutton(FL_RADIO_BUTTON, 
                290, 100, 115, 25,
                "Parsing");
            fl_set_object_lstyle(wids.opt[0], FL_BOLD_STYLE);
            fl_set_object_callback(wids.opt[0], RadioCB, 0);
 
            wids.opt[1] = fl_add_lightbutton(FL_RADIO_BUTTON, 
                290, 130, 115, 25,
                "Output");
            fl_set_object_lstyle(wids.opt[1], FL_BOLD_STYLE);
            fl_set_object_callback(wids.opt[1], RadioCB, 0);
 
            wids.opt[2] = fl_add_lightbutton(FL_RADIO_BUTTON, 
                290, 160, 115, 25,
                "Tracing");
            fl_set_object_lstyle(wids.opt[2], FL_BOLD_STYLE);
            fl_set_object_callback(wids.opt[2], RadioCB, 0);
           
            wids.opt[3] = fl_add_lightbutton(FL_RADIO_BUTTON, 
                290, 190, 115, 25,
                "Animation");
            fl_set_object_lstyle(wids.opt[3], FL_BOLD_STYLE);
            fl_set_object_callback(wids.opt[3], RadioCB, 0);
  
        fl_end_group();
 

        /* Frame for the options groups */
        obj = fl_add_frame(FL_ENGRAVED_FRAME, GRP_X, GRP_Y, GRP_W, GRP_H, "");
 
        /* Startup pixmap that is hidden once an option button is selected */
        pixmap_group();
        wids.current_grp = wids.pixmap_grp;
  
        /* Option groups -- build and hide for now */ 
        parsing_group();
        fl_hide_object(wids.parsing_grp);
        output_group();
        fl_hide_object(wids.output_grp);
        tracing_group();
        fl_hide_object(wids.tracing_grp);
        animation_group();
        fl_hide_object(wids.animation_grp);
        config_group();
        fl_hide_object(wids.config_grp);
               
    fl_end_form();
    
    /* Show this form if redirect option is on */
    wids.text_form = fl_bgn_form(FL_UP_BOX, 560, 350);
 
        wids.text = fl_add_browser(FL_NORMAL_BROWSER, 
            15, 15, 530, 300, ""); 
            
        obj = fl_add_button(FL_NORMAL_BUTTON, 
            245, 320, 70, 25, "Dismiss"); 
            fl_set_object_lstyle(obj, FL_BOLD_STYLE);
            fl_set_object_callback(obj, DismissMessageCB, 0);
        
    fl_end_form(); 

    /* Show this form if there is a fatal error */
    wids.fatal_form = fl_bgn_form(FL_UP_BOX, 560, 350);

        wids.fatal_text = fl_add_browser(FL_NORMAL_BROWSER, 
            15, 15, 530, 300, ""); 
            
        obj = fl_add_button(FL_NORMAL_BUTTON, 
            245, 320, 70, 25, "Dismiss"); 
            fl_set_object_lstyle(obj, FL_BOLD_STYLE);
            fl_set_object_callback(obj, DismissMessageCB, 1);
        
    fl_end_form();
     
     
    wids.about_form = fl_bgn_form(FL_UP_BOX, 400, 270);

        wids.tc = fl_add_box(FL_NO_BOX, 
            150, 30, 120, 40, "xfpovray"); 
            fl_set_object_lsize(wids.tc, FL_HUGE_SIZE);
            fl_set_object_lstyle(wids.tc, FL_BOLDITALIC_STYLE|FL_EMBOSSED_STYLE);
            fl_set_object_lcol(wids.tc, FL_RED);
  
        obj = fl_add_box(FL_NO_BOX, 
            150, 130, 120, 25, "Robert S. Mallozzi"); 
            fl_set_object_lsize(obj, FL_MEDIUM_SIZE);
            fl_set_object_lstyle(obj, FL_BOLD_STYLE);
            fl_set_object_lcol(obj, FL_BLACK);

        obj = fl_add_box(FL_NO_BOX, 
            150, 150, 120, 25, "<mallozzir@cspar.uah.edu>"); 
            fl_set_object_lsize(obj, FL_MEDIUM_SIZE);
            fl_set_object_lstyle(obj, FL_BOLD_STYLE);
            fl_set_object_lcol(obj, FL_BLACK);

        obj = fl_add_box(FL_NO_BOX, 
            150, 170, 120, 25, "http://cspar.uah.edu/~mallozzir/"); 
            fl_set_object_lsize(obj, FL_MEDIUM_SIZE);
            fl_set_object_lstyle(obj, FL_BOLD_STYLE);
            fl_set_object_lcol(obj, FL_BLACK);
                           
        obj = fl_add_button(FL_NORMAL_BUTTON, 175, 220, 70, 25, "Dismiss");
            fl_set_object_callback (obj, DismissAboutCB, 0);
            fl_set_object_lsize(obj, FL_NORMAL_SIZE);
            fl_set_object_lstyle(obj, FL_BOLD_STYLE);

    
        wids.tim = fl_add_timer(FL_HIDDEN_TIMER, 0, 0, 1, 1, "");
        fl_set_timer(wids.tim, TIME);
        fl_set_object_callback(wids.tim, ContinueCB, 0);
 
    fl_end_form();
     
    if (have_geometry == FALSE)
       fl_show_form(wids.main_form, FL_PLACE_CENTERFREE, 
                    FL_FULLBORDER, "xfpovray");
    else {
       fl_set_form_position(wids.main_form, start_x, start_y);
       fl_show_form(wids.main_form, FL_PLACE_GEOMETRY, 
                    FL_FULLBORDER, "xfpovray");
    }
 
    /* Idle callback to monitor the POV-Ray child process */
    fl_set_idle_callback(PovCheckCB, 0); 

    do {
       ret = fl_do_forms();
    } while (ret != NULL);

    fl_hide_form(wids.main_form);
    exit(0);


}
